#region MyJScript - LGPL Copyright (c) 2006 Lionel Lask
//
// This file is part of MyJScript.
//
// MyJScript is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// MyJScript is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with MyJScript; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
//================================================================================
#endregion

using System;
using System.Text;
using System.CodeDom;

namespace MyJScript.Compiler
{
	using System.Collections;

	/// <summary>
	/// Base class to support multiple Jay generated parsers
	/// </summary>
	public abstract class GenericParser
	{
		/// <summary>
		///  Global error count
		/// </summary>
		protected int global_errors;

		/// <summary>
		///  Derived parse method
		/// </summary>
		/// <returns>a namespace with the CodeDom Tree</returns>
		public abstract bool Parse();

		/// <summary>
		/// Emits error messages and increments a global count of them
		/// </summary>
		/// <param name="code"></param>
		/// <param name="desc"></param>
		public void error(int code, string desc)
		{
			Console.Error.WriteLine("Error "+code+": "+ desc);
			global_errors++;
		}

		/// <summary>
		///  Emits error messages with location info.
		/// </summary>
		/// <param name="code">error code</param>
		/// <param name="line">line</param>
        /// <param name="col">col</param>
		/// <param name="text">error text</param>
		public void error(int code, int line, int col, string text)
		{
			Console.Error.WriteLine("(" + line + "," + col +
					   "): Error " + code + ": " + text);
			global_errors++;
		}

		/// <summary>
		/// Derived classes should do their iniatilization here duties
		/// </summary>
		public GenericParser()
		{
			//
			// DO NOTHING: Derived classes should do their iniatilization here duties
			//
		}

		/// <summary>
		/// Used verbose flag or not
		/// </summary>
		protected bool yacc_verbose_flag = false;

		/// <summary>
		/// Verbose output or not
		/// </summary>
		public bool yacc_verbose
		{
			set
			{
				yacc_verbose_flag = value;
			}

			get
			{
				return yacc_verbose_flag;
			}
		}
	}
}



