#region MyJScript - LGPL Copyright (c) 2006 Lionel Lask
//
// This file is part of MyJScript.
//
// MyJScript is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// MyJScript is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with MyJScript; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
//================================================================================
#endregion

using System;
using System.Collections;

namespace MyJScript.Compiler
{
    /// <summary>
    ///   Keeps track of the location in the program
    /// </summary>
    public class Location
    {
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="filename">filename</param>
        /// <param name="row">row</param>
        /// <param name="col">col</param>
        public Location(string filename, int row, int col)
        {
            this.filename = filename;
            this.row = row;
            this.col = col;
        }

        /// <summary>
        /// Display location as a string
        /// </summary>
        /// <returns>a string with the location</returns>
        public override string ToString()
        {
            return Name + ": (" + Row + "," + Col + ")";
        }

        /// <summary>
        /// Current file name
        /// </summary>
        public string Name
        {
            get { return filename; }
        }
        private string filename;

        /// <summary>
        /// Current row in file
        /// </summary>
        public int Row
        {
            get { return row; }
            set { row = value; }
        }
        private int row;

        /// <summary>
        /// Current col in file
        /// </summary>
        public int Col
        {
            get { return col; }
            set { col = value; }
        }
        private int col;
    }
}
