﻿#region MyJScript - LGPL Copyright (c) 2006 Lionel Laské
//
// This file is part of MyJScript.
//
// MyJScript is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// MyJScript is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with MyJScript; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
//================================================================================
#endregion

using System;
using System.CodeDom;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyJScript.Compiler;

namespace MyJScript.Test
{
    /// <summary>
    /// Unit testing for MyJScript compiler
    /// </summary>
    [TestClass]
    public class GeneratorTest
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public GeneratorTest()
        {
        }

        /// <summary>
        /// Empty source
        /// </summary>
        [TestMethod]
        public void Empty()
        {
            Parser p1 = new Parser(null);
            Assert.IsTrue(p1.Parse());
            Assert.IsNotNull(p1.Result);

            CodeNamespace cns1 = p1.Result;
            Assert.AreEqual(1, cns1.Types.Count);

            CodeTypeDeclaration cl1 = cns1.Types[0];
            Assert.AreEqual(1, cl1.Members.Count);

            Assert.AreEqual("Main", cl1.Members[0].Name);


            Parser p2 = new Parser(null);
            Assert.IsTrue(p2.Parse());
            Assert.IsNotNull(p2.Result);

            CodeNamespace cns2 = p2.Result;
            Assert.AreEqual(1, cns2.Types.Count);

            CodeTypeDeclaration cl2 = cns2.Types[0];
            Assert.AreEqual(1, cl2.Members.Count);

            Assert.AreEqual("Main", cl2.Members[0].Name);
        }

        /// <summary>
        /// Test for functions
        /// </summary>
        [TestMethod]
        public void Functions()
        {
            // No parameter
            Parser p1 = new Parser("function hello() {}");
            Assert.IsTrue(p1.Parse());
            Assert.IsNotNull(p1.Result);

            CodeNamespace cns1 = p1.Result;
            Assert.AreEqual(1, cns1.Types.Count);

            CodeTypeDeclaration cl1 = cns1.Types[0];
            Assert.AreEqual(2, cl1.Members.Count);

            Assert.AreEqual("Main", cl1.Members[0].Name);
            Assert.AreEqual("hello", cl1.Members[1].Name);

            CodeMemberMethod m1 = cl1.Members[1] as CodeMemberMethod;
            Assert.IsNotNull(m1);
            Assert.AreEqual(1, m1.Statements.Count);
            Assert.AreEqual(typeof(CodeMethodReturnStatement), m1.Statements[0].GetType());

            // One parameter
            Parser p2 = new Parser("function oneparam(param_1) {}");
            Assert.IsTrue(p2.Parse());
            Assert.IsNotNull(p2.Result);

            CodeNamespace cns2 = p2.Result;
            Assert.AreEqual(1, cns2.Types.Count);

            CodeTypeDeclaration cl2 = cns2.Types[0];
            Assert.AreEqual(2, cl2.Members.Count);

            Assert.AreEqual("Main", cl2.Members[0].Name);
            Assert.AreEqual("oneparam", cl2.Members[1].Name);

            CodeMemberMethod m2 = cl2.Members[1] as CodeMemberMethod;
            Assert.IsNotNull(m2);

            Assert.AreEqual(1, m2.Parameters.Count);
            Assert.AreEqual("param_1", m2.Parameters[0].Name);
            Assert.AreEqual("MJSValue", m2.Parameters[0].Type.BaseType);

            Assert.AreEqual(1, m2.Statements.Count);
            Assert.AreEqual(typeof(CodeMethodReturnStatement), m2.Statements[0].GetType());

            // Two parameters
            Parser p3 = new Parser("function two_param(_p1, p2) {}");
            Assert.IsTrue(p3.Parse());
            Assert.IsNotNull(p3.Result);

            CodeNamespace cns3 = p3.Result;
            Assert.AreEqual(1, cns3.Types.Count);

            CodeTypeDeclaration cl3 = cns3.Types[0];
            Assert.AreEqual(2, cl3.Members.Count);

            Assert.AreEqual("Main", cl3.Members[0].Name);
            Assert.AreEqual("two_param", cl3.Members[1].Name);

            CodeMemberMethod m3 = cl3.Members[1] as CodeMemberMethod;
            Assert.IsNotNull(m3);

            Assert.AreEqual(2, m3.Parameters.Count);
            Assert.AreEqual("_p1", m3.Parameters[0].Name);
            Assert.AreEqual("MJSValue", m3.Parameters[0].Type.BaseType);
            Assert.AreEqual("p2", m3.Parameters[1].Name);
            Assert.AreEqual("MJSValue", m3.Parameters[1].Type.BaseType);

            Assert.AreEqual(1, m3.Statements.Count);
            Assert.AreEqual(typeof(CodeMethodReturnStatement), m3.Statements[0].GetType());

            // Two parameters with same name
            Parser p4 = new Parser("function sameNameParam(p1, p2, P3, p1)\r\n{\r\n}");
            Assert.IsFalse(p4.Parse());
            Assert.AreEqual(1, p4.Errors.Count);
            Assert.AreEqual("MJS0400", p4.Errors[0].ErrorNumber);
            Assert.AreEqual(1, p4.Errors[0].Line);
            Assert.AreEqual(39, p4.Errors[0].Column);

            // Multiple declaration for same function
            Parser p5 = new Parser("function not_unique()\r\n{\r\n}\r\nfunction not_unique(p1)\r\n{\r\n}");
            Assert.IsFalse(p5.Parse());
            Assert.AreEqual(1, p5.Errors.Count);
            Assert.AreEqual("MJS0401", p5.Errors[0].ErrorNumber);
            Assert.AreEqual(4, p5.Errors[0].Line);
            Assert.AreEqual(24, p5.Errors[0].Column);

            // Function don't exist
            Parser p6 = new Parser("write('Hello');\r\nfoo(100, 200);");
            Assert.IsFalse(p6.Parse());
            Assert.AreEqual(1, p6.Errors.Count);
            Assert.AreEqual("MJS0402", p6.Errors[0].ErrorNumber);
            Assert.AreEqual(2, p6.Errors[0].Line);
            Assert.AreEqual(14, p6.Errors[0].Column);

            // Return at global scope
            Parser p7 = new Parser("while(1==1)\r\nreturn;");
            Assert.IsFalse(p7.Parse());
            Assert.AreEqual(1, p7.Errors.Count);
            Assert.AreEqual("MJS0403", p7.Errors[0].ErrorNumber);
            Assert.AreEqual(2, p7.Errors[0].Line);
            Assert.AreEqual(8, p7.Errors[0].Column);
        }

        /// <summary>
        /// Test for global variables
        /// </summary>
        [TestMethod]
        public void GlobalVariables()
        {
            // One variable
            Parser p1 = new Parser("var v;");
            Assert.IsTrue(p1.Parse());
            Assert.IsNotNull(p1.Result);

            CodeNamespace cns1 = p1.Result;
            Assert.AreEqual(1, cns1.Types.Count);

            CodeTypeDeclaration cl1 = cns1.Types[0];
            Assert.AreEqual(2, cl1.Members.Count);

            Assert.AreEqual("v", cl1.Members[0].Name);
            CodeMemberField cf1 = cl1.Members[0] as CodeMemberField;
            Assert.IsNotNull(cf1);
            Assert.AreEqual("MJSValue", cf1.Type.BaseType);
            Assert.AreEqual(MemberAttributes.Public|MemberAttributes.Static, cf1.Attributes);
            Assert.IsNotNull(cf1.InitExpression);

            // Multiple declaration of same var
            Parser p2 = new Parser("var x;\r\nvar x=45;");
            Assert.IsTrue(p2.Parse());
            Assert.IsNotNull(p2.Result);

            CodeNamespace cns2 = p2.Result;
            Assert.AreEqual(1, cns2.Types.Count);

            CodeTypeDeclaration cl2 = cns2.Types[0];
            Assert.AreEqual(2, cl2.Members.Count);

            Assert.AreEqual("x", cl2.Members[0].Name);
            CodeMemberField cf2 = cl2.Members[0] as CodeMemberField;
            Assert.IsNotNull(cf2);
            Assert.AreEqual("MJSValue", cf2.Type.BaseType);
            Assert.AreEqual(MemberAttributes.Public | MemberAttributes.Static, cf2.Attributes);
            Assert.IsNotNull(cf2.InitExpression);

            // Two vars
            Parser p3 = new Parser("var v1;\r\nfunction foo() { }\r\nvar v2='Hello!';");
            Assert.IsTrue(p3.Parse());
            Assert.IsNotNull(p3.Result);

            CodeNamespace cns3 = p3.Result;
            Assert.AreEqual(1, cns3.Types.Count);

            CodeTypeDeclaration cl3 = cns3.Types[0];
            Assert.AreEqual(4, cl3.Members.Count);

            Assert.AreEqual("v2", cl3.Members[0].Name);
            CodeMemberField cf3 = cl3.Members[0] as CodeMemberField;
            Assert.IsNotNull(cf3);
            Assert.AreEqual("MJSValue", cf3.Type.BaseType);
            Assert.AreEqual(MemberAttributes.Public | MemberAttributes.Static, cf3.Attributes);
            Assert.IsNotNull(cf3.InitExpression);
            Assert.AreEqual("v1", cl3.Members[1].Name);
            CodeMemberField cf4 = cl3.Members[1] as CodeMemberField;
            Assert.IsNotNull(cf4);
            Assert.AreEqual("MJSValue", cf4.Type.BaseType);
            Assert.AreEqual(MemberAttributes.Public | MemberAttributes.Static, cf4.Attributes);
            Assert.IsNotNull(cf4.InitExpression);
        }

        /// <summary>
        /// Test for local variables
        /// </summary>
        [TestMethod]
        public void LocalVariables()
        {
            // One variable
            Parser p1 = new Parser("function foo() { var v; }");
            Assert.IsTrue(p1.Parse());
            Assert.IsNotNull(p1.Result);

            CodeNamespace cns1 = p1.Result;
            Assert.AreEqual(1, cns1.Types.Count);

            CodeTypeDeclaration cl1 = cns1.Types[0];
            Assert.AreEqual(2, cl1.Members.Count);
            Assert.AreEqual("foo", cl1.Members[1].Name);

            CodeMemberMethod m1 = cl1.Members[1] as CodeMemberMethod;
            Assert.IsNotNull(m1);
            Assert.IsTrue(m1.Statements.Count >= 2);

            CodeVariableDeclarationStatement v1 = m1.Statements[0] as CodeVariableDeclarationStatement;
            Assert.IsNotNull(v1);
            Assert.AreEqual("v", v1.Name);
            Assert.AreEqual("MJSValue", v1.Type.BaseType);
            Assert.IsNull(v1.InitExpression);

            // Nested variable but same scope
            Parser p2 = new Parser("function foo() { var v; while(2==0) { var w=3; } }");
            Assert.IsTrue(p2.Parse());
            Assert.IsNotNull(p2.Result);

            CodeNamespace cns2 = p2.Result;
            Assert.AreEqual(1, cns2.Types.Count);

            CodeTypeDeclaration cl2 = cns2.Types[0];
            Assert.AreEqual(2, cl2.Members.Count);
            Assert.AreEqual("foo", cl2.Members[1].Name);

            CodeMemberMethod m2 = cl2.Members[1] as CodeMemberMethod;
            Assert.IsNotNull(m2);
            Assert.IsTrue(m2.Statements.Count >= 3);

            CodeVariableDeclarationStatement v2 = m2.Statements[0] as CodeVariableDeclarationStatement;
            Assert.IsNotNull(v2);
            Assert.AreEqual("w", v2.Name);
            Assert.AreEqual("MJSValue", v2.Type.BaseType);
            Assert.IsNull(v2.InitExpression);
            CodeVariableDeclarationStatement v3 = m2.Statements[1] as CodeVariableDeclarationStatement;
            Assert.IsNotNull(v3);
            Assert.AreEqual("v", v3.Name);
            Assert.AreEqual("MJSValue", v3.Type.BaseType);
            Assert.IsNull(v3.InitExpression);

            // Local scope conflict with global scope
            Parser p3 = new Parser("var v='MJS'; function foo() { var v='Go!'; }");
            Assert.IsTrue(p3.Parse());
            Assert.IsNotNull(p3.Result);

            CodeNamespace cns3 = p3.Result;
            Assert.AreEqual(1, cns3.Types.Count);

            CodeTypeDeclaration cl3 = cns3.Types[0];
            Assert.AreEqual(3, cl3.Members.Count);
            Assert.AreEqual("foo", cl3.Members[2].Name);

            CodeMemberMethod m3 = cl3.Members[2] as CodeMemberMethod;
            Assert.IsNotNull(m3);
            Assert.IsTrue(m3.Statements.Count >= 2);

            CodeVariableDeclarationStatement v4 = m3.Statements[0] as CodeVariableDeclarationStatement;
            Assert.IsNotNull(v4);
            Assert.AreEqual("v", v4.Name);
            Assert.AreEqual("MJSValue", v4.Type.BaseType);
            Assert.IsNull(v4.InitExpression);

            // Local scope conflict with parameter, don't declare variable
            Parser p4 = new Parser("function foo(a) { var a=100; }");
            Assert.IsTrue(p4.Parse());
            Assert.IsNotNull(p4.Result);

            CodeNamespace cns4 = p4.Result;
            Assert.AreEqual(1, cns4.Types.Count);

            CodeTypeDeclaration cl4 = cns4.Types[0];
            Assert.AreEqual(2, cl4.Members.Count);
            Assert.AreEqual("foo", cl4.Members[1].Name);

            CodeMemberMethod m4 = cl4.Members[1] as CodeMemberMethod;
            Assert.IsNotNull(m4);
            Assert.IsTrue(m1.Statements.Count >= 1);

            CodeVariableDeclarationStatement v5 = m4.Statements[0] as CodeVariableDeclarationStatement;
            Assert.IsNull(v5);
        }
    }
}
