﻿#region MyJScript - LGPL Copyright (c) 2006 Lionel Laské
//
// This file is part of MyJScript.
//
// MyJScript is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// MyJScript is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with MyJScript; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
//================================================================================
#endregion

using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyJScript.Runtime;

namespace MyJScript.Test
{
    /// <summary>
    /// Unit test for MJSValue class
    /// </summary>
    [TestClass]
    public class MJSValueTest
    {
        // Constructor.
        public MJSValueTest()
        {
        }

        /// <summary>
        /// Test constructor.
        /// </summary>
        [TestMethod]
        public void Constructor()
        {
            MJSValue v1 = new MJSValue();
            Assert.IsTrue(v1.IsNull());

            MJSValue v2 = new MJSValue(null);
            Assert.IsTrue(v2.IsNull());

            MJSValue v3 = new MJSValue(30);
            Assert.AreEqual((float)30, v3.Value);

            MJSValue v4 = new MJSValue("MyJScript");
            Assert.AreEqual("MyJScript", v4.Value);
        }

        /// <summary>
        /// Test operator +.
        /// </summary>
        [TestMethod]
        public void Add()
        {
            MJSValue v1 = new MJSValue();
            MJSValue res = v1 + v1;
            Assert.IsTrue(res.IsNull());

            MJSValue v3 = new MJSValue(12);
            res = v1 + v3;
            Assert.IsTrue(res.IsNull());
            res = v3 + v1;
            Assert.IsTrue(res.IsNull());

            v3 = new MJSValue("Liogo");
            res = v1 + v3;
            Assert.IsTrue(res.IsNull());
            res = v3 + v1;
            Assert.IsTrue(res.IsNull());

            MJSValue v4 = new MJSValue(3.1f);
            MJSValue v5 = new MJSValue(4.2f);
            res = v4 + v5;
            Assert.AreEqual(7.3f.ToString(), res.Value.ToString());
            res = v5 + v4;
            Assert.AreEqual(7.3f.ToString(), res.Value.ToString());

            v4 = new MJSValue("He");
            v5 = new MJSValue("llo");
            res = v4 + v5;
            Assert.AreEqual("Hello", res.Value);
            res = v5 + v4;
            Assert.AreEqual("lloHe", res.Value);

            v4 = new MJSValue("My");
            v5 = new MJSValue(100);
            res = v4 + v5;
            Assert.AreEqual("My100", res.Value);
            res = v5 + v4;
            Assert.AreEqual("100My", res.Value);

            v4 = new MJSValue("2");
            v5 = new MJSValue(100);
            res = v4 + v5;
            Assert.AreEqual("2100", res.Value);
            res = v5 + v4;
            Assert.AreEqual("1002", res.Value);
        }

        /// <summary>
        /// Test operator *.
        /// </summary>
        [TestMethod]
        public void Product()
        {
            MJSValue v1 = new MJSValue();
            MJSValue res = v1 * v1;
            Assert.IsTrue(res.IsNull());

            MJSValue v3 = new MJSValue(12);
            res = v1 * v3;
            Assert.IsTrue(res.IsNull());
            res = v3 * v1;
            Assert.IsTrue(res.IsNull());

            v3 = new MJSValue("Liogo");
            res = v1 * v3;
            Assert.IsTrue(res.IsNull());
            res = v3 * v1;
            Assert.IsTrue(res.IsNull());

            MJSValue v4 = new MJSValue(3f);
            MJSValue v5 = new MJSValue(4.2f);
            res = v4 * v5;
            Assert.AreEqual(12.6f.ToString(), res.Value.ToString());
            res = v5 * v4;
            Assert.AreEqual(12.6f.ToString(), res.Value.ToString());

            v4 = new MJSValue("He");
            v5 = new MJSValue("llo");
            res = v4 * v5;
            Assert.IsTrue(res.IsNull());
            res = v5 * v4;
            Assert.IsTrue(res.IsNull());

            v4 = new MJSValue("My");
            v5 = new MJSValue(100);
            res = v4 * v5;
            Assert.IsTrue(res.IsNull());
            res = v5 * v4;
            Assert.IsTrue(res.IsNull());

            v4 = new MJSValue("2");
            v5 = new MJSValue(100);
            res = v4 * v5;
            Assert.AreEqual("200", res.Value.ToString());
            res = v5 * v4;
            Assert.AreEqual("200", res.Value.ToString());
        }

        /// <summary>
        /// Test operator -.
        /// </summary>
        [TestMethod]
        public void Minus()
        {
            MJSValue v1 = new MJSValue();
            MJSValue res = v1 - v1;
            Assert.IsTrue(res.IsNull());

            MJSValue v3 = new MJSValue(12);
            res = v1 - v3;
            Assert.IsTrue(res.IsNull());
            res = v3 - v1;
            Assert.IsTrue(res.IsNull());

            v3 = new MJSValue("Liogo");
            res = v1 - v3;
            Assert.IsTrue(res.IsNull());
            res = v3 - v1;
            Assert.IsTrue(res.IsNull());

            MJSValue v4 = new MJSValue(4.2f);
            MJSValue v5 = new MJSValue(3f);
            res = v4 - v5;
            Assert.AreEqual(1.2f.ToString(), res.Value.ToString());
            res = v5 - v4;
            Assert.AreEqual((-1.2f).ToString(), res.Value.ToString());

            v4 = new MJSValue("He");
            v5 = new MJSValue("llo");
            res = v4 - v5;
            Assert.IsTrue(res.IsNull());
            res = v5 - v4;
            Assert.IsTrue(res.IsNull());

            v4 = new MJSValue("My");
            v5 = new MJSValue(100);
            res = v4 - v5;
            Assert.IsTrue(res.IsNull());
            res = v5 - v4;
            Assert.IsTrue(res.IsNull());

            v4 = new MJSValue("2");
            v5 = new MJSValue(100);
            res = v4 - v5;
            Assert.AreEqual("-98", res.Value.ToString());
            res = v5 - v4;
            Assert.AreEqual("98", res.Value.ToString());
        }

        /// <summary>
        /// Test operator /.
        /// </summary>
        [TestMethod]
        public void Divide()
        {
            MJSValue v1 = new MJSValue();
            MJSValue res = v1 / v1;
            Assert.IsTrue(res.IsNull());

            MJSValue v3 = new MJSValue(12);
            res = v1 / v3;
            Assert.IsTrue(res.IsNull());
            res = v3 / v1;
            Assert.IsTrue(res.IsNull());

            v3 = new MJSValue("MJScript");
            res = v1 / v3;
            Assert.IsTrue(res.IsNull());
            res = v3 / v1;
            Assert.IsTrue(res.IsNull());

            MJSValue v4 = new MJSValue(12f);
            MJSValue v5 = new MJSValue(3f);
            res = v4 / v5;
            Assert.AreEqual(4f.ToString(), res.Value.ToString());
            res = v5 / v4;
            Assert.AreEqual(0.25f.ToString(), res.Value.ToString());

            v4 = new MJSValue("He");
            v5 = new MJSValue("llo");
            res = v4 / v5;
            Assert.IsTrue(res.IsNull());
            res = v5 / v4;
            Assert.IsTrue(res.IsNull());

            v4 = new MJSValue("My");
            v5 = new MJSValue(100);
            res = v4 / v5;
            Assert.IsTrue(res.IsNull());
            res = v5 / v4;
            Assert.IsTrue(res.IsNull());

            v4 = new MJSValue("4");
            v5 = new MJSValue(2);
            res = v4 / v5;
            Assert.AreEqual("2", res.Value.ToString());
            res = v5 / v4;
            Assert.AreEqual(0.5f.ToString(), res.Value.ToString());

            v4 = new MJSValue(100f);
            v5 = new MJSValue(0f);
            res = v4 / v5;
            Assert.AreEqual(float.PositiveInfinity.ToString(), res.Value.ToString());
        }

        /// <summary>
        /// Test Equals.
        /// </summary>
        [TestMethod]
        public void Equals()
        {
            MJSValue v1 = new MJSValue();
            Assert.IsTrue(v1.Equals(v1));

            Assert.IsFalse(v1.Equals(null));

            MJSValue v2 = new MJSValue();
            Assert.IsTrue(v1.Equals(v2));
            Assert.IsTrue(v2.Equals(v1));

            MJSValue v3 = new MJSValue("DNG");
            Assert.IsFalse(v1.Equals(v3));
            Assert.IsFalse(v3.Equals(v1));
            Assert.IsFalse(v3.Equals("DNG"));

            MJSValue v4 = new MJSValue("DNG");
            Assert.IsTrue(v3.Equals(v4));
            Assert.IsTrue(v4.Equals(v3));

            MJSValue v5 = new MJSValue("MJS");
            Assert.IsFalse(v4.Equals(v5));
            Assert.IsFalse(v5.Equals(v4));

            MJSValue v6 = new MJSValue(100.5f);
            Assert.IsFalse(v1.Equals(v6));
            Assert.IsFalse(v6.Equals(v1));
            Assert.IsFalse(v5.Equals(v6));
            Assert.IsFalse(v6.Equals(v5));

            MJSValue v7 = new MJSValue(100.5f);
            Assert.IsTrue(v7.Equals(v6));
            Assert.IsTrue(v6.Equals(v7));

            MJSValue v8 = new MJSValue(69f);
            MJSValue v9 = new MJSValue(69);
            Assert.IsTrue(v8.Equals(v9));
            Assert.IsTrue(v9.Equals(v8));
        }

        /// <summary>
        /// Test Equality.
        /// </summary>
        [TestMethod]
        public void Equality()
        {
            Assert.IsTrue((MJSValue)null == (MJSValue)null);

            MJSValue v1 = new MJSValue();
            Assert.IsTrue(v1 == v1);
            Assert.IsFalse(v1 == null);
            Assert.IsFalse(null == v1);

            MJSValue v2 = new MJSValue();
            Assert.IsTrue(v1 == v2);
            Assert.IsTrue(v2 == v1);

            MJSValue v3 = new MJSValue("DNG");
            Assert.IsFalse(v1 == v3);
            Assert.IsFalse(v3 == v1);

            MJSValue v4 = new MJSValue("DNG");
            Assert.IsTrue(v3 == v4);
            Assert.IsTrue(v4 == v3);

            MJSValue v5 = new MJSValue("MJS");
            Assert.IsFalse(v4 == v5);
            Assert.IsFalse(v5 == v4);

            MJSValue v6 = new MJSValue(100.5f);
            Assert.IsFalse(v1 == v6);
            Assert.IsFalse(v6 == v1);
            Assert.IsFalse(v5 == v6);
            Assert.IsFalse(v6 == v5);

            MJSValue v7 = new MJSValue(100.5f);
            Assert.IsTrue(v7 == v6);
            Assert.IsTrue(v6 == v7);

            MJSValue v8 = new MJSValue(69f);
            MJSValue v9 = new MJSValue(69);
            Assert.IsTrue(v8 == v9);
            Assert.IsTrue(v9 == v8);

            MJSValue v10 = new MJSValue("69");
            Assert.IsTrue(v9 == v10);
            Assert.IsTrue(v10 == v9);
        }

        /// <summary>
        /// Test Equality.
        /// </summary>
        [TestMethod]
        public void Inequality()
        {
            Assert.IsFalse((MJSValue)null != (MJSValue)null);

            MJSValue v1 = new MJSValue();
            Assert.IsFalse(v1 != v1);
            Assert.IsTrue(v1 != null);
            Assert.IsTrue(null != v1);

            MJSValue v2 = new MJSValue();
            Assert.IsFalse(v1 != v2);
            Assert.IsFalse(v2 != v1);

            MJSValue v3 = new MJSValue("DNG");
            Assert.IsTrue(v1 != v3);
            Assert.IsTrue(v3 != v1);

            MJSValue v4 = new MJSValue("DNG");
            Assert.IsFalse(v3 != v4);
            Assert.IsFalse(v4 != v3);

            MJSValue v5 = new MJSValue("MJS");
            Assert.IsTrue(v4 != v5);
            Assert.IsTrue(v5 != v4);

            MJSValue v6 = new MJSValue(100.5f);
            Assert.IsTrue(v1 != v6);
            Assert.IsTrue(v6 != v1);
            Assert.IsTrue(v5 != v6);
            Assert.IsTrue(v6 != v5);

            MJSValue v7 = new MJSValue(100.5f);
            Assert.IsFalse(v7 != v6);
            Assert.IsFalse(v6 != v7);

            MJSValue v8 = new MJSValue(69f);
            MJSValue v9 = new MJSValue(69);
            Assert.IsFalse(v8 != v9);
            Assert.IsFalse(v9 != v8);

            MJSValue v10 = new MJSValue("69");
            Assert.IsFalse(v9 != v10);
            Assert.IsFalse(v10 != v9);
        }

        /// <summary>
        /// Test CompareTo.
        /// </summary>
        [TestMethod]
        public void CompareTo()
        {
            MJSValue v1 = new MJSValue();
            try
            {
                v1.CompareTo("Bla");
                Assert.Fail();
            }
            catch (ArgumentException)
            {
            }
            try
            {
                v1.CompareTo(null);
                Assert.Fail();
            }
            catch (ArgumentNullException)
            {
            }

            Assert.AreEqual(0, v1.CompareTo(v1));

            MJSValue v2 = new MJSValue(100f);
            Assert.AreEqual(0, v2.CompareTo(v2));
            try
            {
                v2.CompareTo(v1);
                Assert.Fail();
            }
            catch (ArgumentException)
            {
            }
            try
            {
                v1.CompareTo(v2);
                Assert.Fail();
            }
            catch (ArgumentException)
            {
            }
            MJSValue v3 = new MJSValue("Go!");
            Assert.AreEqual(0, v3.CompareTo(v3));

            MJSValue v4 = new MJSValue(128f);
            Assert.IsTrue(v2.CompareTo(v4) < 0);
            Assert.IsTrue(v4.CompareTo(v2) > 0);
            MJSValue v5 = new MJSValue("ZZZ");
            Assert.IsTrue(v3.CompareTo(v5) < 0);
            Assert.IsTrue(v5.CompareTo(v3) > 0);
            MJSValue v6 = new MJSValue("10");
            Assert.IsTrue(v6.CompareTo(v5) < 0);
            Assert.IsTrue(v5.CompareTo(v6) > 0);
            Assert.IsTrue(v4.CompareTo(v6) > 0);
            Assert.IsTrue(v6.CompareTo(v4) < 0);
            MJSValue v7 = new MJSValue("1000");
            Assert.IsTrue(v7.CompareTo(v5) < 0);
            Assert.IsTrue(v5.CompareTo(v7) > 0);
            Assert.IsTrue(v4.CompareTo(v7) < 0);
            Assert.IsTrue(v7.CompareTo(v4) > 0);
        }

        /// <summary>
        /// Test GreaterThan.
        /// </summary>
        [TestMethod]
        public void GreaterThan()
        {
            MJSValue v1 = new MJSValue();
            try
            {
                Assert.IsFalse(v1 > null);
            }
            catch (ArgumentNullException)
            {
            }
            try
            {
                Assert.IsFalse(null > v1);
            }
            catch (ArgumentNullException)
            {
            }
            try
            {
                Assert.IsFalse(v1 > v1);
            }
            catch (NullReferenceException)
            {
            }

            MJSValue v2 = new MJSValue(100f);
            Assert.IsFalse(v2 > v2);
            MJSValue v3 = new MJSValue("Go!");
            Assert.IsFalse(v3 > v3);

            MJSValue v4 = new MJSValue(128f);
            Assert.IsFalse(v2 > v4);
            Assert.IsTrue(v4 > v2);
            MJSValue v5 = new MJSValue("ZZZ");
            Assert.IsFalse(v3 > v5);
            Assert.IsTrue(v5 > v3);
            MJSValue v6 = new MJSValue("10");
            Assert.IsFalse(v6 > v5);
            Assert.IsTrue(v5 > v6);
            Assert.IsTrue(v4 > v6);
            Assert.IsFalse(v6 > v4);
            MJSValue v7 = new MJSValue("1000");
            Assert.IsFalse(v7 > v5);
            Assert.IsTrue(v5 > v7);
            Assert.IsFalse(v4 > v7);
            Assert.IsTrue(v7 > v4);
        }

        /// <summary>
        /// Test GreaterOrEqualThan.
        /// </summary>
        [TestMethod]
        public void GreaterOrEqualThan()
        {
            Assert.IsTrue((MJSValue)null >= (MJSValue)null);

            MJSValue v1 = new MJSValue();
            try
            {
                Assert.IsFalse(v1 >= null);
            }
            catch (ArgumentNullException)
            {
            }
            try
            {
                Assert.IsFalse(null >= v1);
            }
            catch (ArgumentNullException)
            {
            }
            try
            {
                Assert.IsTrue(v1 >= v1);
            }
            catch (NullReferenceException)
            {
            }

            MJSValue v2 = new MJSValue(100f);
            Assert.IsTrue(v2 >= v2);
            MJSValue v3 = new MJSValue("Go!");
            Assert.IsTrue(v3 >= v3);

            MJSValue v4 = new MJSValue(128f);
            Assert.IsFalse(v2 >= v4);
            Assert.IsTrue(v4 >= v2);
            MJSValue v5 = new MJSValue("ZZZ");
            Assert.IsFalse(v3 >= v5);
            Assert.IsTrue(v5 >= v3);

            MJSValue v6 = new MJSValue("10");
            Assert.IsFalse(v6 >= v5);
            Assert.IsTrue(v5 >= v6);
            Assert.IsTrue(v4 >= v6);
            Assert.IsFalse(v6 >= v4);
            MJSValue v7 = new MJSValue("1000");
            Assert.IsFalse(v7 >= v5);
            Assert.IsTrue(v5 >= v7);
            Assert.IsFalse(v4 >= v7);
            Assert.IsTrue(v7 >= v4);
        }

        /// <summary>
        /// Test LessThan.
        /// </summary>
        [TestMethod]
        public void LessThan()
        {
            MJSValue v1 = new MJSValue();
            try
            {
                Assert.IsFalse(v1 < null);
            }
            catch (ArgumentNullException)
            {
            }
            try
            {
                Assert.IsFalse(null < v1);
            }
            catch (ArgumentNullException)
            {
            }
            try
            {
                Assert.IsFalse(v1 < v1);
            }
            catch (NullReferenceException)
            {
            }

            MJSValue v2 = new MJSValue(100f);
            Assert.IsFalse(v2 < v2);
            MJSValue v3 = new MJSValue("Go!");
            Assert.IsFalse(v3 < v3);

            MJSValue v4 = new MJSValue(128f);
            Assert.IsTrue(v2 < v4);
            Assert.IsFalse(v4 < v2);
            MJSValue v5 = new MJSValue("ZZZ");
            Assert.IsTrue(v3 < v5);
            Assert.IsFalse(v5 < v3);
            MJSValue v6 = new MJSValue("10");
            Assert.IsTrue(v6 < v5);
            Assert.IsFalse(v5 < v6);
            Assert.IsFalse(v4 < v6);
            Assert.IsTrue(v6 < v4);
            MJSValue v7 = new MJSValue("1000");
            Assert.IsTrue(v7 < v5);
            Assert.IsFalse(v5 < v7);
            Assert.IsTrue(v4 < v7);
            Assert.IsFalse(v7 < v4);
        }

        /// <summary>
        /// Test LessOrEqualThan.
        /// </summary>
        [TestMethod]
        public void LessOrEqualThan()
        {
            Assert.IsTrue((MJSValue)null <= (MJSValue)null);

            MJSValue v1 = new MJSValue();
            try
            {
                Assert.IsFalse(v1 <= null);
            }
            catch (ArgumentNullException)
            {
            }
            try
            {
                Assert.IsFalse(null <= v1);
            }
            catch (ArgumentNullException)
            {
            }
            try
            {
                Assert.IsTrue(v1 <= v1);
            }
            catch (NullReferenceException)
            {
            }

            MJSValue v2 = new MJSValue(100f);
            Assert.IsTrue(v2 <= v2);
            MJSValue v3 = new MJSValue("Go!");
            Assert.IsTrue(v3 <= v3);

            MJSValue v4 = new MJSValue(128f);
            Assert.IsTrue(v2 <= v4);
            Assert.IsFalse(v4 <= v2);
            MJSValue v5 = new MJSValue("ZZZ");
            Assert.IsTrue(v3 <= v5);
            Assert.IsFalse(v5 <= v3);

            MJSValue v6 = new MJSValue("10");
            Assert.IsTrue(v6 <= v5);
            Assert.IsFalse(v5 <= v6);
            Assert.IsFalse(v4 <= v6);
            Assert.IsTrue(v6 <= v4);
            MJSValue v7 = new MJSValue("1000");
            Assert.IsTrue(v7 <= v5);
            Assert.IsFalse(v5 <= v7);
            Assert.IsTrue(v4 <= v7);
            Assert.IsFalse(v7 <= v4);
        }

        /// <summary>
        /// Test ToString().
        /// </summary>
        [TestMethod]
        public new void ToString()
        {
            MJSValue v1 = new MJSValue();
            Assert.AreEqual("undefined", v1.ToString());

            MJSValue v2 = new MJSValue(345f);
            Assert.AreEqual("345", v2.ToString());

            MJSValue v3 = new MJSValue("DDJ");
            Assert.AreEqual("DDJ", v3.ToString());
        }

        /// <summary>
        /// Test GetHashCode().
        /// </summary>
        [TestMethod]
        public new void GetHashCode()
        {
            MJSValue v1 = new MJSValue(13f);
            MJSValue v2 = new MJSValue("Hello");
            Assert.AreNotEqual(v1.GetHashCode(), v2.GetHashCode());
            MJSValue v3 = new MJSValue(3.14159f);
            Assert.AreNotEqual(v1.GetHashCode(), v3.GetHashCode());
            MJSValue v4 = new MJSValue("Bye");
            Assert.AreNotEqual(v2.GetHashCode(), v4.GetHashCode());
            MJSValue v5 = new MJSValue(13f);
            MJSValue v6 = new MJSValue("Hello");
            Assert.AreNotEqual(v1.GetHashCode(), v5.GetHashCode());
            Assert.AreNotEqual(v3.GetHashCode(), v6.GetHashCode());
        }
    }
}
