﻿#region MyJScript - LGPL Copyright (c) 2006 Lionel Laské
//
// This file is part of MyJScript.
//
// MyJScript is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// MyJScript is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with MyJScript; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
//================================================================================
#endregion

using System;
using System.Text;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyJScript.Compiler;

namespace MyJScript.Test
{
    /// <summary>
    /// Unit tests for Parser.
    /// </summary>
    [TestClass]
    public class ParserTest
    {
        public ParserTest()
        {
        }

        /// <summary>
        /// Parse source code and print error.
        /// </summary>
        /// <param name="source">source file</param>
        /// <returns>false if error</returns>
        private bool Parse(string source)
        {
            Parser p = new Parser(source);
            bool res = p.Parse();
            if (!res)
            {
                for (int i = 0; i < p.Errors.Count; i++)
                    Console.Error.WriteLine(p.Errors[i].ToString());
            }
            return res;
        }

        /// <summary>
        /// Test empty source file.
        /// </summary>
        [TestMethod]
        public void Empty()
        {
            Parser p1 = new Parser(null);
            Assert.IsTrue(p1.Parse());
            Assert.AreEqual(0, p1.Errors.Count);

            Parser p2 = new Parser("");
            Assert.IsTrue(p2.Parse());
            Assert.AreEqual(0, p2.Errors.Count);

            Parser p3 = new Parser("   \r\n    \t\r\n");
            Assert.IsTrue(p3.Parse());
            Assert.AreEqual(0, p3.Errors.Count);
        }

        /// <summary>
        /// Test single statement.
        /// </summary>
        [TestMethod]
        public void SingleStatement()
        {
            Assert.IsTrue(Parse("var a;"));
            Assert.IsTrue(Parse("var _r='Hello!';"));
            Assert.IsTrue(Parse("while(i==0) i=i+1;"));
            Assert.IsTrue(Parse("write(12);"));
        }

        /// <summary>
        /// Test multiple statements.
        /// </summary>
        [TestMethod]
        public void MultipleStatement()
        {
            Assert.IsTrue(Parse(@"
                var i = 0;
                while (i<3)
                    i = i + 1;
            "));

            Assert.IsTrue(Parse(@"
                function hello() {
                    write(""Hello world!"");
                }

                hello();
            "));
        }

        /// <summary>
        /// Test when parsing error.
        /// </summary>
        [TestMethod]
        public void Error()
        {
            Parser p1 = new Parser("var #");
            Assert.IsFalse(p1.Parse());
            Assert.AreEqual(1, p1.Errors.Count);
            Console.Error.WriteLine(p1.Errors[0].ToString());
            Assert.AreEqual("MJS0100", p1.Errors[0].ErrorNumber);
            Assert.AreEqual(1, p1.Errors[0].Line);
            Assert.AreEqual(5, p1.Errors[0].Column);
            
            Parser p2 = new Parser("var a");
            Assert.IsFalse(p2.Parse());
            Assert.AreEqual(1, p2.Errors.Count);
            Assert.AreEqual("MJS0200", p2.Errors[0].ErrorNumber);
            Assert.AreEqual(1, p2.Errors[0].Line);
            Assert.AreEqual(6, p2.Errors[0].Column);

            Parser p3 = new Parser("var b;\r\nb = var;");
            Assert.IsFalse(p3.Parse());
            Assert.AreEqual(1, p3.Errors.Count);
            Assert.AreEqual("MJS0300", p3.Errors[0].ErrorNumber);
            Assert.AreEqual(2, p3.Errors[0].Line);
            Assert.AreEqual(8, p3.Errors[0].Column);
        }
    }
}
